import board
import busio
import time
import usb_hid
from adafruit_hid.keyboard import Keyboard
from adafruit_hid.keycode import Keycode
import adafruit_vcnl4040
import json
import neopixel # Import the neopixel library

# --- Load Configuration from config.json ---
config = {}
try:
    with open("config.json", "r") as config_file:
        config = json.load(config_file)
except OSError as e:
    print(f"Could not open config.json, using default values. Error: {e}")
    # Default configuration if file is missing or corrupted
    config = {
        "proximity_threshold": 10,
        "triggered_key": "TAB",
        "key_press_duration_seconds": 0.1,
        "loop_delay_seconds": 0.25,
        "only_closer_trigger": True,
        "enable_debug_prints": False,
        "enable_neopixel_feedback": False, # Default: NeoPixel feedback off
        "neopixel_color_rgb": [0, 10, 0]   # Default: Dim green
    }

# Map string keycodes from JSON to actual Keycode objects
keycode_map = {
    "A": Keycode.A, "B": Keycode.B, "C": Keycode.C, "D": Keycode.D, "E": Keycode.E,
    "F": Keycode.F, "G": Keycode.G, "H": Keycode.H, "I": Keycode.I, "J": Keycode.J,
    "K": Keycode.K, "L": Keycode.L, "M": Keycode.M, "N": Keycode.N, "O": Keycode.O,
    "P": Keycode.P, "Q": Keycode.Q, "R": Keycode.R, "S": Keycode.S, "T": Keycode.T,
    "U": Keycode.U, "V": Keycode.V, "W": Keycode.W, "X": Keycode.X, "Y": Keycode.Y,
    "Z": Keycode.Z, "ONE": Keycode.ONE, "TWO": Keycode.TWO, "THREE": Keycode.THREE,
    "FOUR": Keycode.FOUR, "FIVE": Keycode.FIVE, "SIX": Keycode.SIX, "SEVEN": Keycode.SEVEN,
    "EIGHT": Keycode.EIGHT, "NINE": Keycode.NINE, "ZERO": Keycode.ZERO,
    "F1": Keycode.F1, "F2": Keycode.F2, "F3": Keycode.F3, "F4": Keycode.F4,
    "F5": Keycode.F5, "F6": Keycode.F6, "F7": Keycode.F7, "F8": Keycode.F8,
    "F9": Keycode.F9, "F10": Keycode.F10, "F11": Keycode.F11, "F12": Keycode.F12,
    "ENTER": Keycode.ENTER, "RETURN": Keycode.RETURN,
    "ESCAPE": Keycode.ESCAPE, "BACKSPACE": Keycode.BACKSPACE, "TAB": Keycode.TAB,
    "SPACE": Keycode.SPACE, "MINUS": Keycode.MINUS, "EQUALS": Keycode.EQUALS,
    "LEFT_BRACKET": Keycode.LEFT_BRACKET, "RIGHT_BRACKET": Keycode.RIGHT_BRACKET,
    "BACKSLASH": Keycode.BACKSLASH, "SEMICOLON": Keycode.SEMICOLON,
    "QUOTE": Keycode.QUOTE, "GRAVE_ACCENT": Keycode.GRAVE_ACCENT,
    "COMMA": Keycode.COMMA, "PERIOD": Keycode.PERIOD, "FORWARD_SLASH": Keycode.FORWARD_SLASH,
    "CAPS_LOCK": Keycode.CAPS_LOCK, "SCROLL_LOCK": Keycode.SCROLL_LOCK,
    "PRINT_SCREEN": Keycode.PRINT_SCREEN, "PAUSE": Keycode.PAUSE, "INSERT": Keycode.INSERT,
    "HOME": Keycode.HOME, "PAGE_UP": Keycode.PAGE_UP, "DELETE": Keycode.DELETE,
    "END": Keycode.END, "PAGE_DOWN": Keycode.PAGE_DOWN, "RIGHT_ARROW": Keycode.RIGHT_ARROW,
    "LEFT_ARROW": Keycode.LEFT_ARROW, "DOWN_ARROW": Keycode.DOWN_ARROW, "UP_ARROW": Keycode.UP_ARROW,
    "KEYPAD_FORWARD_SLASH": Keycode.KEYPAD_FORWARD_SLASH,
    "KEYPAD_ASTERISK": Keycode.KEYPAD_ASTERISK, "KEYPAD_MINUS": Keycode.KEYPAD_MINUS,
    "KEYPAD_PLUS": Keycode.KEYPAD_PLUS, "KEYPAD_ENTER": Keycode.KEYPAD_ENTER,
    "KEYPAD_ONE": Keycode.KEYPAD_ONE, "KEYPAD_TWO": Keycode.KEYPAD_TWO,
    "KEYPAD_THREE": Keycode.KEYPAD_THREE, "KEYPAD_FOUR": Keycode.KEYPAD_FOUR,
    "KEYPAD_FIVE": Keycode.KEYPAD_FIVE, "KEYPAD_SIX": Keycode.KEYPAD_SIX,
    "KEYPAD_SEVEN": Keycode.KEYPAD_SEVEN, "KEYPAD_EIGHT": Keycode.EIGHT,
    "KEYPAD_NINE": Keycode.KEYPAD_NINE, "KEYPAD_ZERO": Keycode.ZERO,
    "KEYPAD_PERIOD": Keycode.KEYPAD_PERIOD,
    "LEFT_CONTROL": Keycode.LEFT_CONTROL, "LEFT_SHIFT": Keycode.LEFT_SHIFT,
    "LEFT_ALT": Keycode.LEFT_ALT, "LEFT_GUI": Keycode.LEFT_GUI,
    "RIGHT_CONTROL": Keycode.RIGHT_CONTROL, "RIGHT_SHIFT": Keycode.RIGHT_SHIFT,
    "RIGHT_ALT": Keycode.RIGHT_ALT, "RIGHT_GUI": Keycode.RIGHT_GUI,
}

# --- Apply Configuration ---
PROXIMITY_THRESHOLD = config["proximity_threshold"]
TRIGGERED_KEYCODE = keycode_map.get(config["triggered_key"].upper(), Keycode.TAB)
KEY_PRESS_DURATION = config["key_press_duration_seconds"]
LOOP_DELAY = config["loop_delay_seconds"]
ONLY_CLOSER_TRIGGER = config["only_closer_trigger"]
ENABLE_DEBUG_PRINTS = config["enable_debug_prints"]
ENABLE_NEOPIXEL_FEEDBACK = config["enable_neopixel_feedback"] # New NeoPixel setting
NEOPIXEL_COLOR = tuple(config["neopixel_color_rgb"]) # New NeoPixel color as a tuple (R,G,B)

print(f"Configuration loaded: PROXIMITY_THRESHOLD={PROXIMITY_THRESHOLD}, TRIGGERED_KEY={config['triggered_key']}, KEY_PRESS_DURATION={KEY_PRESS_DURATION}, LOOP_DELAY={LOOP_DELAY}, ONLY_CLOSER_TRIGGER={ONLY_CLOSER_TRIGGER}, ENABLE_DEBUG_PRINTS={ENABLE_DEBUG_PRINTS}, ENABLE_NEOPIXEL_FEEDBACK={ENABLE_NEOPIXEL_FEEDBACK}, NEOPIXEL_COLOR={NEOPIXEL_COLOR}")

# --- Initialize Hardware ---
try:
    i2c = busio.I2C(board.SCL1, board.SDA1)
except ValueError as e:
    print(f"Error initializing I2C bus: {e}. Make sure the STEMMA QT cable is plugged in correctly.")
    while True:
        time.sleep(1)

vcnl4040 = adafruit_vcnl4040.VCNL4040(i2c)
keyboard = Keyboard(usb_hid.devices)

# Initialize NeoPixel if enabled in config
if ENABLE_NEOPIXEL_FEEDBACK:
    # board.NEOPIXEL is the built-in NeoPixel on QT Py RP2040
    # 1 is the number of pixels, brightness is 0.0 to 1.0
    pixel = neopixel.NeoPixel(board.NEOPIXEL, 1, brightness=0.5, auto_write=False)
    pixel.fill((0, 0, 0)) # Start with NeoPixel off
    pixel.show()
else:
    # If not enabled, make sure any existing pixel is off
    try:
        pixel = neopixel.NeoPixel(board.NEOPIXEL, 1, brightness=0.0, auto_write=False)
        pixel.fill((0,0,0))
        pixel.show()
    except Exception: # In case neopixel board pin is not defined or similar
        pass


# --- Main Loop ---
last_proximity = vcnl4040.proximity

while True:
    try:
        current_proximity = vcnl4040.proximity

        if ENABLE_DEBUG_PRINTS:
            print(f"Current proximity: {current_proximity}")

        triggered = False
        if ONLY_CLOSER_TRIGGER:
            if (current_proximity - last_proximity) >= PROXIMITY_THRESHOLD:
                triggered = True
        else:
            if abs(current_proximity - last_proximity) >= PROXIMITY_THRESHOLD:
                triggered = True

        if triggered:
            # Turn on NeoPixel if enabled
            if ENABLE_NEOPIXEL_FEEDBACK:
                pixel.fill(NEOPIXEL_COLOR)
                pixel.show()

            keyboard.press(TRIGGERED_KEYCODE)
            print(f"{config['triggered_key']} key pressed!")
            time.sleep(KEY_PRESS_DURATION)
            keyboard.release(TRIGGERED_KEYCODE)
            print(f"{config['triggered_key']} key released!")

            # Turn off NeoPixel if enabled
            if ENABLE_NEOPIXEL_FEEDBACK:
                pixel.fill((0, 0, 0)) # Turn off NeoPixel (black color)
                pixel.show()

        last_proximity = current_proximity
        time.sleep(LOOP_DELAY)

    except OSError as e:
        print("Sensor error:", e)
        # Attempt to re-initialize I2C if there's an OSError
        try:
            i2c = busio.I2C(board.SCL1, board.SDA1)
            vcnl4040 = adafruit_vcnl4040.VCNL4040(i2c)
            print("Sensor re-initialized after error.")
        except ValueError:
            print("Failed to re-initialize sensor. Check connections.")
            time.sleep(1)
